/**
 * Available methods:
 * * - All/Wildcard
 * n - None
 * c - Create
 * r - Read
 * u - Update
 * d - Delete
 * x - Execute
 */
export type Method = '*' | 'n' | 'c' | 'r' | 'u' | 'd' | 'x';
/**
 * A pathed event broken down into its core components for easy processing
 * PathedEvent Structure: `module/path:scope:method`
 */
export type PathedEvent = {
    /** Original pathed string */
    full: string;
    /** First directory in path */
    module: string;
    /** Entire path, including the module */
    fullPath: string;
    /** Path, excluding the module */
    path: string;
    /** List of methods */
    methods: Method[];
    /** All/Wildcard specified */
    all: boolean;
    /** None specified */
    none: boolean;
    /** Create method specified */
    create: boolean;
    /** Read method specified */
    read: boolean;
    /** Update method specified */
    update: boolean;
    /** Delete method specified */
    delete: boolean;
    /** Allow running target */
    execute: boolean;
};
/**
 * Combine multiple pathed events into one parsed object. Longest path takes precedent, but all subsequent methods are
 * combined until a "none" is reached
 *
 * @param {string | PathedEvent} paths PathedEvents as strings or pre-parsed
 * @return {PathedEvent} Final combined permission
 */
export declare function combinePathedEvents(...paths: (string | PathedEvent)[]): PathedEvent;
/**
 * Squash 2 sets of paths & return true if any overlap is found
 *
 * @param {string | PathedEvent | (string | PathedEvent)[]} target Array of PathedEvents as strings or pre-parsed
 * @param {string} anyPath List of PathedEvents the target must match at least one of
 * @return {boolean} Whether there is any overlap
 */
export declare function hasPath(target: string | PathedEvent | (string | PathedEvent)[], ...anyPath: (string | PathedEvent)[]): boolean;
/**
 * Same as `hasPath` but raises an error if there is no overlap
 *
 * @param {string | string[]} target Array of PathedEvents as strings or pre-parsed
 * @param {string} paths List of PathedEvents the target must match at least one of
 */
export declare function hasPathFatal(target: string | string[], ...paths: string[]): void;
/**
 * Create pathed event string from its components
 *
 * @param {string | string[]} path Event path
 * @param {Method} methods Event method
 * @return {string} Final combined PathedEvent as a string
 */
export declare function pathedEvent(path: string | string[], ...methods: Method[]): string;
/**
 * Parse a PathedEvent string into its components
 * @param {string | PathedEvent} path PathedEvent as a string
 * @return {PathedEvent} PathedEvent parsed into its components
 */
export declare function parsePathedEvent(path: string | PathedEvent): PathedEvent;
export type Listener = (event: PathedEvent, ...args: any[]) => any;
export type Unsubscribe = () => void;
export interface IPathedEventEmitter {
    emit(event: string, ...args: any[]): void;
    off(listener: Listener): void;
    on(event: string, listener: Listener): Unsubscribe;
    once(event: string, listener?: Listener): Promise<any>;
    relayEvents(emitter: PathedEventEmitter): void;
}
/**
 * Event emitter that uses paths allowing listeners to listen to different combinations of modules, paths & methods
 */
export declare class PathedEventEmitter implements IPathedEventEmitter {
    private listeners;
    emit(event: string | PathedEvent, ...args: any[]): void;
    off(listener: Listener): void;
    on(event: string | string[], listener: Listener): Unsubscribe;
    once(event: string | string[], listener?: Listener): Promise<any>;
    relayEvents(emitter: IPathedEventEmitter): void;
}
//# sourceMappingURL=pathed-events.d.ts.map